import os
import shutil
import wget
import zipfile
import requests
from subprocess import run, PIPE

def atualiza_chromedriver():
    """
    Função que checa a versão do Chrome driver e, caso a versão instalada esteja desatualizada, atualiza para a mais recente.
    Isto é feito baixando o zip e extraindo o arquivo .exe na pasta bin.
    O arquivo desatualizado do Chromedriver será excluído assim como o arquivo zip da versão mais recente.
    """
    bin_folder_path = "Arquivo_Robo/bin"
    os.makedirs(bin_folder_path, exist_ok=True)

    def get_actual_chromedriver_path(folder):
        """Acha o caminho do ChromeDriver atual"""
        pattern = os.path.join(
            folder, "chromedriver.exe"
        )  # Explicitly look for .exe file
        if os.path.exists(pattern):
            return pattern
        return None

    def get_latest_chromedriver_version():
        """Pega a última versão do disponível do ChromeDriver"""
        url = "https://googlechromelabs.github.io/chrome-for-testing/last-known-good-versions.json"
        try:
            response = requests.get(url)
            response.raise_for_status()
            data = response.json()
            return data["channels"]["Stable"]["version"]
        except requests.RequestException as e:
            raise RuntimeError(f"Failed to fetch ChromeDriver version: {e}")
        except:
            raise RuntimeError(
                "Failed to parse JSON data or find the expected structure."
            )

    def get_local_chromedriver_version(driver_path):
        """Retorna a versão do ChromeDriver local"""
        if driver_path and os.path.exists(driver_path):
            result = run(
                [driver_path, "--version"], stdout=PIPE, stderr=PIPE, text=True
            )
            if result.returncode == 0:
                return result.stdout.split(" ")[1].strip()
        return None

    chrome_driver_path = get_actual_chromedriver_path(bin_folder_path)
    local_version = get_local_chromedriver_version(chrome_driver_path)
    latest_version = get_latest_chromedriver_version()

    if not local_version or local_version.split(".")[0] != latest_version.split(".")[0]:
        """Checa se a versão do ChromeDriver local é diferente da versão mais recente analisando apenas o primeiro número da versão.
        Exemplo 1:

        Versão mais recente: 114.2.5735.90
        Versão local: 114.0.5735.90

        RESULTADO = NÃO ATUALIZA


        Exemplo 2:

        Versão mais recente: 131.2.5735.90
        Versão local: 129.2.5735.90

        RESULTADO = ATUALIZA
        """
        print("Atualizando o chromedriver")

        download_url = f"https://storage.googleapis.com/chrome-for-testing-public/{latest_version}/win64/chromedriver-win64.zip"

        # Download the zip file using the URL built above
        latest_driver_zip = wget.download(download_url, "chromedriver.zip")

        # Ensure the bin directory exists
        destination_folder = os.path.join("Arquivo_Robo/bin")
        os.makedirs(destination_folder, exist_ok=True)

        # Move the downloaded ZIP file into the bin directory
        destination_zip = os.path.join(
            destination_folder, os.path.basename(latest_driver_zip)
        )
        shutil.move(latest_driver_zip, destination_zip)

        # Extract the ZIP file into the bin directory
        with zipfile.ZipFile(destination_zip, "r") as zip_ref:
            zip_ref.extractall(destination_folder)

        # Path of the extracted folder
        extracted_folder = os.path.join(destination_folder, "chromedriver-win64")

        # Move "chromedriver.exe" to the "bin" folder
        chromedriver_path = os.path.join(extracted_folder, "chromedriver.exe")

        if os.path.exists(chromedriver_path):
            if chrome_driver_path:  # Only remove if it exists
                os.remove(chrome_driver_path)  # Remove the existing file
            shutil.move(chromedriver_path, destination_folder)

        # Delete the extracted folder
        shutil.rmtree(extracted_folder)

        # Delete the ZIP file after extraction
        os.remove(destination_zip)
        print(
            f'\nChromedriver atualizado com sucesso para a versão {latest_version}.\nCertifique-se de que a versão do Google Chrome é a mais atual para evitar problema do tipo "SessionNotCreatedException".'
        )



import os
import wget
import time
import shutil
import base64
import zipfile
import requests
import numpy as np
import pandas as pd
import tkinter as tk
from subprocess import run, PIPE
from tkinter import messagebox, Tk
from payconpy.utils.utils import *
from payconpy.fpython.fpython import *
from datetime import datetime, timedelta
from msal import ConfidentialClientApplication

def wait_for_downloads(directory, timeout=600):
    """
    Waits until there are no files with .crdownload or .part extensions in the directory,
    indicating that all downloads have completed.

    Parameters:
    - directory: The directory to monitor for downloads.
    - timeout: Maximum time to wait for downloads to complete, in seconds.

    Returns:
    - True if all downloads are completed within the timeout period, False otherwise.
    """
    start_time = time.time()
    while time.time() - start_time < timeout:
        files = os.listdir(directory)
        if not any(file.endswith((".crdownload", ".part")) for file in files):
            return True
        time.sleep(1)
    return False

def apaga_e_printa(frase, color="white"):
    os.system("cls" if os.name == "nt" else "clear")
    faz_log(frase, color=color)  # Assuming faz_log is defined elsewhere

def progress_bar(current, total, bar_length=40):
    """
    Generates a text-based progress bar.

    Args:
        current (int): The current progress.
        total (int): The value representing 100% progress.
        bar_length (int): The length of the progress bar (default is 20).

    Returns:
        str: A string representing the progress bar.
    """
    if total == 0:
        return "Progress: [ERROR - Total cannot be zero]"

    progress = current / total
    if progress > 1:
        progress = 1  # Cap the progress at 100%

    filled_length = int(bar_length * progress)
    bar = "█" * filled_length + "-" * (bar_length - filled_length)
    percentage = progress * 100
    return f"PROGRESSO: [{bar}] {percentage:.1f}%"

def show_success_popup(titulo, mensagem):
    """Displays a success popup window."""
    root = tk.Tk()
    root.withdraw()  # Hide the main window
    messagebox.showinfo(titulo, mensagem)
    root.destroy()

def find_elements_with_retry(driver, wait_time, locator, retries=3):
    """Finds elements and retries if a StaleElementReferenceException occurs."""
    from selenium.webdriver.support.ui import WebDriverWait
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.common.exceptions import StaleElementReferenceException

    for attempt in range(retries):
        try:
            elements = WebDriverWait(driver, wait_time).until(
                EC.presence_of_all_elements_located(locator)
            )
            return elements
        except StaleElementReferenceException:
            if attempt == retries - 1:
                raise

def find_element_text_with_retry(driver, wait_time, locator, retries=3):
    """Finds element text and retries on StaleElementReferenceException."""
    from selenium.webdriver.support.ui import WebDriverWait
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.common.exceptions import StaleElementReferenceException

    for attempt in range(retries):
        try:
            element = WebDriverWait(driver, wait_time).until(
                EC.presence_of_element_located(locator)
            )
            return element.text
        except StaleElementReferenceException:
            if attempt == retries - 1:
                raise

def download_file(file_id, access_token, download_path):
    headers = {"Authorization": f"Bearer {access_token}"}
    download_url = f"https://graph.microsoft.com/v1.0/me/drive/items/{file_id}/content"
    response = requests.get(download_url, headers=headers, stream=True)

    if response.status_code == 200:
        with open(download_path, "wb") as file:
            for chunk in response.iter_content(chunk_size=8192):
                file.write(chunk)
        print(f"Downloaded {download_path}")
    else:
        print(f"Failed to download file: {response.json()}")

# List files in the folder and download each file
def list_and_download_files(
    access_token, folder_id, download_dir, scope, client_id, client_secret, tenant_id
):
    headers = {"Authorization": f"Bearer {access_token}"}
    response = requests.get(
        f"https://graph.microsoft.com/v1.0/me/drive/items/{folder_id}/children",
        headers=headers,
    )

    if response.status_code == 200:
        items = response.json().get("value", [])
        for item in items:
            if "file" in item:  # Check if the item is a file
                file_id = item["id"]
                file_name = item["name"]
                file_path = os.path.join(download_dir, file_name)
                download_file(file_id, access_token, file_path)
    elif response.status_code == 401:  # Token expired, retry with a new token
        app = ConfidentialClientApplication(
            client_id,
            authority=f"https://login.microsoftonline.com/{tenant_id}",
            client_credential=client_secret,
        )
        result = app.acquire_token_for_client(scopes=scope)
        if "access_token" in result:
            list_and_download_files(
                result["access_token"],
                folder_id,
                download_dir,
                scope,
                client_id,
                client_secret,
                tenant_id,
            )
        else:
            print("Failed to obtain access token.")
            print(result.get("error_description", "No error description available."))
            print(result.get("claims", "No claims available."))
    else:
        print(f"Failed to list files: {response.json()}")

def imprime_iframe(driver):
    iframe_id = driver.execute_script("return window.frameElement ? window.frameElement.id : '';")
    iframe_name = driver.execute_script("return window.frameElement ? window.frameElement.name : '';")
    
    faz_log(f"ID do iframe atual: {iframe_id}")
    faz_log(f"Nome do iframe atual: {iframe_name}")
    return

def retorna_id_iframe_para_variavel(driver):
    iframe_id = driver.execute_script("return window.frameElement ? window.frameElement.id : '';")

    return iframe_id


import os
import re
import time
import base64
import requests
from msal import ConfidentialClientApplication


def baixar_arquivos_onedrive(
    client_id,
    client_secret,
    tenant_id,
    caminho_pasta,
    folder_link,
    comentarios=False
    # baixar_tudo=False,
    # pasta_registros="base",
    # nome_coluna="caso",
    # nome_pasta="historico",
):
    '''
    A função original criada para o SPC possui uma lógica que verifica se o arquivo já foi baixado
    Esta parte foi removida para não ser preciso passar diversas variáveis desnecessárias
    A parte removida se encontrava no else após o if baixar_tudo
    '''
    if comentarios:
        print("Iniciando o download do banco de dados dos processos encontrados")

    local_download_path = caminho_pasta
    # dataset_path = f"{pasta_registros}/{nome_pasta}.xlsx"  # Path to your dataset

    # Carregando o nome do arquivos já baixados
    # if comentarios:
        # print("Carregando o nome do arquivos já baixados")
    # df = pd.read_excel(dataset_path)
    # downloaded_files = set(
    #     df[nome_coluna]
    # )  # Assuming 'nome' column contains the file names

    # Gerando token de acesso
    if comentarios:
        print("Gerando token de acesso")
    token_url = f"https://login.microsoftonline.com/{tenant_id}/oauth2/v2.0/token"
    token_data = {
        "grant_type": "client_credentials",
        "client_id": client_id,
        "client_secret": client_secret,
        "scope": "https://graph.microsoft.com/.default",
    }
    token_response = requests.post(token_url, data=token_data)
    token_response.raise_for_status()
    access_token = token_response.json().get("access_token")

    # Codificando o link da pasta no formato base64
    if comentarios:
        print("Codificando o link da pasta no formato base64")
    encoded_link = base64.b64encode(folder_link.encode()).decode()
    encoded_link = encoded_link.replace("/", "_").replace("+", "-").replace("=", "")

    # Acessando a pasta do OneDrive
    if comentarios:
        print("Acessando a pasta do OneDrive")
    headers = {"Authorization": f"Bearer {access_token}"}
    shared_link_url = (
        f"https://graph.microsoft.com/v1.0/shares/u!{encoded_link}/driveItem"
    )
    shared_link_response = requests.get(shared_link_url, headers=headers)
    shared_link_response.raise_for_status()
    drive_item = shared_link_response.json()

    # Listando os arquivos na pasta do OneDrive
    if comentarios:
        print("Codificando o link da pasta no formato base64")
    folder_id = drive_item["id"]
    files_url = f'https://graph.microsoft.com/v1.0/drives/{drive_item["parentReference"]["driveId"]}/items/{folder_id}/children'
    files_response = requests.get(files_url, headers=headers)
    files_response.raise_for_status()
    files = files_response.json().get("value", [])

    # Baixando arquivos pendentes
    if comentarios:
        print("Baixando arquivos pendentes")
    if not os.path.exists(local_download_path):
        os.makedirs(local_download_path)

    with requests.Session() as session:
        session.headers.update({"Authorization": f"Bearer {access_token}"})

        for file in files:
            file_name = file["name"]
            # if baixar_tudo:
            if comentarios:
                print(f"Baixando arquivo entitulado {file_name}")

            for attempt in range(3):  # Retry up to 3 times
                try:
                    file_id = file["id"]
                    download_url = f'https://graph.microsoft.com/v1.0/drives/{drive_item["parentReference"]["driveId"]}/items/{file_id}/content'
                    file_response = session.get(
                        download_url, timeout=30
                    )  # Increased timeout
                    file_response.raise_for_status()

                    with open(
                        os.path.join(local_download_path, file_name), "wb"
                    ) as f:
                        f.write(file_response.content)

                    if comentarios:
                        print(f"Arquivo baixado com sucesso!")
                    break  # Exit loop if successful
                except requests.exceptions.ConnectionError as e:
                    if attempt < 2:  # Check if this is not the last attempt
                        time.sleep(5)  # Wait for 5 seconds before retrying
                        continue  # Retry
                    else:
                        raise e  # Re-raise the exception after all retries
    if comentarios:
        print(
            f"\nProcesso de atualização do banco de dados concluido!"
        )

def generate_access_token(client_id, client_secret, tenant_id):
    token_url = f"https://login.microsoftonline.com/{tenant_id}/oauth2/v2.0/token"
    token_data = {
        "grant_type": "client_credentials",
        "client_id": client_id,
        "client_secret": client_secret,
        "scope": "https://graph.microsoft.com/.default",
    }
    token_response = requests.post(token_url, data=token_data)
    token_response.raise_for_status()
    return token_response.json().get("access_token")

def get_site_drive_folder_ids(folder_link, access_token):
    # Ensure folder_link is a string
    if not isinstance(folder_link, str):
        raise ValueError(
            "Expected a string for 'folder_link', got: " + str(type(folder_link))
        )

    # Encode the folder link to base64
    encoded_link = base64.b64encode(folder_link.encode()).decode()
    encoded_link = encoded_link.replace("/", "_").replace("+", "-").replace("=", "")

    headers = {"Authorization": f"Bearer {access_token}"}

    # Step 1: Get the drive item details from the shared link
    shared_link_url = (
        f"https://graph.microsoft.com/v1.0/shares/u!{encoded_link}/driveItem"
    )
    response = requests.get(shared_link_url, headers=headers)
    response.raise_for_status()
    drive_item = response.json()

    # Extract the drive ID, folder ID, and site ID from the drive item
    drive_id = drive_item["parentReference"]["driveId"]
    folder_id = drive_item["id"]
    site_id = drive_item["parentReference"]["siteId"]

    return site_id, drive_id, folder_id

def upload_file_to_onedrive(
    client_id, client_secret, tenant_id, file_path, folder_link
):
    """
    Uploads a file to a specific folder in OneDrive using Microsoft Graph API.

    Parameters:
    - client_id (str): Azure AD application client ID.
    - client_secret (str): Azure AD application client secret.
    - tenant_id (str): Azure AD tenant ID.
    - file_path (str): The local path to the file to upload.
    - folder_link (str): The shared link to the folder in OneDrive.

    Returns:
    - None: No output to terminal.
    """
    # Generate access token using the provided function
    access_token = generate_access_token(client_id, client_secret, tenant_id)

    # Get site_id, drive_id, and folder_id using the existing function
    site_id, drive_id, folder_id = get_site_drive_folder_ids(folder_link, access_token)

    # Get file name from the path
    file_name = os.path.basename(file_path)

    # Construct the upload URL
    upload_url = f"https://graph.microsoft.com/v1.0/sites/{site_id}/drives/{drive_id}/items/{folder_id}:/{file_name}:/content"

    headers = {
        "Authorization": "Bearer " + access_token,
        "Content-Type": "application/octet-stream",
    }


from setuptools import setup, find_packages  # 🔥 Import find_packages()

setup(
    name="heraldopy",
    version="1.0.8",  # Update version when making changes
    author="Heraldo Almeida",
    author_email="heraldo.almeida@gmail.com",
    description="A set of automation tools",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/heraldopy",
    packages=find_packages(),  # 🔥 This will now work correctly
    install_requires=[
        "requests",
        "msal",
        "wget",
        "numpy",
        "pandas",
        "tk",
    ],
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
